@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -40,25 +40,25 @@ Some key features
   * Various handy *tools*, plus the ability to make your own.
   * Matlab-style *cell notation* to mark code sections (by starting a line
     with '##').
-  * Highly customizable using the menu.
+  * Highly customizable.
 
 """
-
+    
 # Set version number
-__version__ = '3.1.1'
+__version__ = '3.3'
 
-
- # Fix for issue 137 (apply before importing PySide, just to be safe)
 import os
-os.environ['LIBOVERLAY_SCROLLBAR'] = '0'
-
 import sys
+
 from pyzolib import ssdf, paths
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 
 # Import yoton as an absolute package
 from iep import yotonloader
 
+# Import language/translation tools
+from iep.util.locale import translate, setLanguage
+
 
 ## Define some functions
 
@@ -164,72 +164,7 @@ def saveConfig():
         ssdf.save( os.path.join(appDataDir, "config.ssdf"), config )
 
 
-class Translation(str):
-    """ Derives from str class. The translate function returns an instance
-    of this class and assigns extra atrributes:
-      * original: the original text passed to the translation
-      * tt: the tooltip text 
-      * key: the original text without tooltip (used by menus as a key)
-    
-    We adopt a simple system to include tooltip text in the same
-    translation as the label text. By including ":::" in the text,
-    the text after that identifier is considered the tooltip.
-    The text returned by the translate function is always the 
-    string without tooltip, but the text object has an attribute
-    "tt" that stores the tooltip text. In this way, if you do not
-    use this feature or do not know about this feature, everything
-    keeps working as expected.
-    """
-    pass
 
-def translate(context, text, disambiguation=None):  
-    def splitMainAndTt(s):
-        if ':::' in s:
-            parts = s.split(':::', 1)
-            return parts[0].rstrip(), parts[1].lstrip()
-        else:
-            return s, ''
-    
-    # Get translation and split tooltip
-    newtext = QtCore.QCoreApplication.translate(context, text, disambiguation)
-    s, tt = splitMainAndTt(newtext)
-    # Create translation object (string with extra attributes)
-    translation = Translation(s)
-    translation.original = text
-    translation.tt = tt
-    translation.key = splitMainAndTt(text)[0].strip()
-    return translation
-
-
-def setLanguage(languageId):
-    """ setLanguage(languageId)
-    Set the language for the app. Loads qt and iep translations.
-    Returns the QLocale instance to pass to the main widget.
-    """
-    
-    # Derive name, locale, and locale language name
-    languageName = QtCore.QLocale.languageToString(languageId)
-    locale = QtCore.QLocale(languageId)
-    localeName = locale.name().split('_')[0]
-    
-    # Get paths were language files are
-    qtTransPath = str(QtCore.QLibraryInfo.location(
-                    QtCore.QLibraryInfo.TranslationsPath))
-    iepTransPath = os.path.join(iepDir, 'resources')
-     
-    # Set Qt translations
-    # Note that the translator instances must be stored
-    # Note that the load() method is very forgiving with the file name
-    QtCore._translators = []
-    for what, where in [('qt', qtTransPath),('iep', iepTransPath)]:
-        trans = QtCore.QTranslator()
-        success = trans.load(what + '_' + localeName + '.tr', where)
-        print('loading %s %s: %s' % (what, languageName, ['failed', 'ok'][success]))
-        if success:
-            QtGui.QApplication.installTranslator(trans)
-            QtCore._translators.append(trans)
-    
-    return locale
 
 
 def startIep():
@@ -244,19 +179,11 @@ def startIep():
     # Set to be aware of the systems native colors, fonts, etc.
     QtGui.QApplication.setDesktopSettingsAware(True)
     
-    #Prevent loading plugins form the users' plugin dir since
-    #this may cause multiple versions of the Qt library to be loaded
-    #at once, which will conflict
-    QtGui.QApplication.setLibraryPaths([])
-    
     # Instantiate the application
     QtGui.qApp = QtGui.QApplication([])
     
     # Choose language, get locale
-    # todo: Turn back on so we can find all strings that we should translate
-    # Also implement way to allow user to select the language
-    locale = None
-    #locale = setLanguage(QtCore.QLocale.Dutch)
+    locale = setLanguage(config.settings.language)
     
     # Create main window, using the selected locale
     frame = MainWindow(None, locale)
@@ -285,12 +212,19 @@ _saveConfigFile = True
 config = ssdf.new()
 loadConfig()
 
+# Get license info
+# Yes, you could insert your custom dict here! But who would you be fooling?
+from iep.iepcore.license import get_license_info
+license = get_license_info()
+del get_license_info
+
 # Init default style name (set in main.restoreIepState())
 defaultQtStyleName = ''
 
 # Init pyzo_mode. In pyzo_mode, IEP will use a different logo and possibly
 # expose certain features in the future.
 pyzo_mode = False
+distro_name = None
 
 # Init default exe for the executable (can be set, e.g. by Pyzo)
 _defaultInterpreterExe = None
