@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -155,7 +155,7 @@ def getEnvFromKernelInfo(info):
     
     pythonPath = info.pythonPath
     
-    # Set default pythonPath (replace only first occurance of $PYTHONPATH
+    # Set default pythonPath (replace only first occurrence of $PYTHONPATH
     ENV_PP = os.environ.get('PYTHONPATH','')
     pythonPath = pythonPath.replace('$PYTHONPATH', '\n'+ENV_PP+'\n', 1)
     pythonPath = pythonPath.replace('$PYTHONPATH', '')
@@ -349,7 +349,10 @@ class KernelBroker:
         # Start process
         self._process = subprocess.Popen(   command, shell=True, 
                                             env=env, cwd=cwd,
-                            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+                                            stdin=subprocess.PIPE,  # Fixes issue 165
+                                            stdout=subprocess.PIPE, 
+                                            stderr=subprocess.STDOUT 
+                                        )
         
         # Set timeout for connection, i.e. after how much time of 
         # unresponsive ness is the kernel found to be running extension code
@@ -691,9 +694,15 @@ class StreamReader(threading.Thread):
         self._strm_raw = strm_raw
         self._strm_broker = strm_broker
         self.deamon = True
+        self._exit = False
+    
+    def stop(self, timeout=1.0):
+        self._exit = True
+        self.join(timeout)
     
     def run(self):
-        while True:
+        while not self._exit:
+            time.sleep(0.001)
             # Read any stdout/stderr messages and route them via yoton.
             msg = self._process.stdout.readline() # <-- Blocks here
             if not isinstance(msg, str):
@@ -704,9 +713,7 @@ class StreamReader(threading.Thread):
                 pass # Channel is closed
             # Process dead?
             if not msg:# or self._process.poll() is not None:
-                break
-            # Sleep
-            time.sleep(0.1)
+                break            
         #self._strm_broker.send('streamreader exit\n')
     
 
