@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -19,8 +19,8 @@ from pyzolib import ssdf, paths
 
 import iep
 from iep.iepcore.icons import IconArtist
-from iep.codeeditor.qt import QtCore, QtGui
-
+from pyzolib.qt import QtCore, QtGui
+from iep.iepcore.splash import SplashWidget
 
 
 class MainWindow(QtGui.QMainWindow):
@@ -28,11 +28,13 @@ class MainWindow(QtGui.QMainWindow):
     def __init__(self, parent=None, locale=None):
         QtGui.QMainWindow.__init__(self, parent)
         
+        self._closeflag = 0  # Used during closing/restarting
+        
         # Init window title and application icon
         # Set title to something nice. On Ubuntu 12.10 this text is what
         # is being shown at the fancy title bar (since it's not properly 
         # updated)
-        self.setWindowTitle("The Interactive Editor for Python")
+        self.setMainTitle()
         loadAppIcons()
         self.setWindowIcon(iep.icon)
         
@@ -41,23 +43,20 @@ class MainWindow(QtGui.QMainWindow):
         self.resize(800, 600) # default size
         self.restoreGeometry()
         
-        # Change background of main window to create a splash-screen-efefct
-        iconImage = 'pyzologo256.png' if iep.pyzo_mode else 'ieplogo256.png'
-        iconImage = os.path.join(iep.iepDir, 'resources','appicons', iconImage)
-        iconImage = iconImage.replace(os.path.sep, '/') # Fix for Windows
-        self.setStyleSheet( """QMainWindow { 
-                            background-color: #268bd2;
-                            background-image: url("%s");
-                            background-repeat: no-repeat;
-                            background-position: center;
-                            }
-                            """ % iconImage)
+        # Show splash screen (we need to set our color too)
+        w = SplashWidget(self, distro=iep.distro_name)
+        self.setCentralWidget(w)
+        self.setStyleSheet("QMainWindow { background-color: #268bd2;}")
         
         # Show empty window and disable updates for a while
         self.show()
         self.paintNow()
         self.setUpdatesEnabled(False)
         
+        # Determine timeout for showing splash screen
+        splash_timeout = time.time() + 1.0
+        splash_timeout += 0.0 if iep.license else 2.0
+        
         # Set locale of main widget, so that qt strings are translated
         # in the right way
         if locale:
@@ -77,12 +76,19 @@ class MainWindow(QtGui.QMainWindow):
         # Set window atrributes
         self.setAttribute(QtCore.Qt.WA_AlwaysShowToolTips, True)
         
-        # Load icons
+        # Load icons and fonts
         loadIcons()
+        loadFonts()
         
         # Set qt style and test success
         self.setQtStyle(None) # None means init!
         
+        # Hold the splash screen if needed
+        while time.time() < splash_timeout:
+            QtGui.qApp.flush()
+            QtGui.qApp.processEvents()
+            time.sleep(0.05)
+        
         # Populate the window (imports more code)
         self._populate()
         
@@ -98,11 +104,11 @@ class MainWindow(QtGui.QMainWindow):
         # Load basic tools if new user
         if iep.config.state.newUser and not iep.config.state.loadedTools:
             iep.toolManager.loadTool('iepsourcestructure')
-            iep.toolManager.loadTool('iepprojectmanager')
+            iep.toolManager.loadTool('iepfilebrowser')
         
         # Present user with wizard if he/she is new.
         if iep.config.state.newUser:
-            from iep.iepcore.iepwizard import IEPWizard
+            from iep.util.iepwizard import IEPWizard
             w = IEPWizard(self)
             w.show() # Use show() instead of exec_() so the user can interact with IEP
         
@@ -185,6 +191,34 @@ class MainWindow(QtGui.QMainWindow):
                 iep.toolManager.loadTool(toolId)
     
     
+    def setMainTitle(self, path=None):
+        """ Set the title of the main window, by giving a file path.
+        """
+        if not path:
+            # Plain title
+            title = "Interactive Editor for Python"
+        else:
+            # Title with a filename
+            name = os.path.basename(path)
+            if os.path.isfile(path):
+                pass
+            elif name == path:
+                path = 'no location on disk'
+            else:
+                pass  # We hope the given path is informative
+            # Set title
+            tmp = { 'fileName':name, 'filename':name, 'name':name,
+                    'fullPath':path, 'fullpath':path, 'path':path }
+            title = iep.config.advanced.titleText.format(**tmp)
+        
+        # Add license info
+        if iep.license:
+            title += ' - licensed to {name}'.format(**iep.license)
+        
+        # Set
+        self.setWindowTitle(title)
+    
+    
     def saveWindowState(self):
         """ Save:
             * which tools are loaded 
@@ -287,6 +321,11 @@ class MainWindow(QtGui.QMainWindow):
             if stylename2 != stylename:
                 useStandardStyle = True
         
+        # Check if this style exist, set to default otherwise
+        styleNames = [name.lower() for name in QtGui.QStyleFactory.keys()]
+        if stylename2.lower() not in styleNames:
+            stylename2 = iep.defaultQtStyleName
+        
         # Try changing the style
         qstyle = QtGui.qApp.setStyle(stylename2)
         
@@ -304,35 +343,68 @@ class MainWindow(QtGui.QMainWindow):
     def closeEvent(self, event):
         """ Override close event handler. """
         
+        # Are we restaring?
+        restarting = time.time() - self._closeflag < 1.0
+        
         # Save settings
         iep.saveConfig()
         
         # Proceed with closing...
         result = iep.editors.closeAll()
         if not result:
-            self._didClose = False
+            self._closeflag = False
             event.ignore()
             return
         else:
-            self._didClose = True
-            event.accept()
+            self._closeflag = True
+            #event.accept()  # Had to comment on Windows+py3.3 to prevent error
         
         # Proceed with closing shells
         iep.localKernelManager.terminateAll()
         for shell in iep.shells:
             shell._context.close()
         
-        # Close as normal
+        # Close tools
+        for toolname in iep.toolManager.getLoadedTools():
+            tool = iep.toolManager.getTool(toolname) 
+            tool.close()
+        
+        # Stop all threads (this should really only be daemon threads)
+        import threading
+        for thread in threading.enumerate():
+            if hasattr(thread, 'stop'):
+                try:
+                    thread.stop(0.1)
+                except Exception:
+                    pass
+        
+#         # Wait for threads to die ... 
+#         # This should not be necessary, but I used it in the hope that it
+#         # would prevent the segfault on Python3.3. It didn't.
+#         timeout = time.time() + 0.5
+#         while threading.activeCount() > 1 and time.time() < timeout:
+#             time.sleep(0.1)
+#         print('Number of threads alive:', threading.activeCount())
+        
+        # Proceed as normal
         QtGui.QMainWindow.closeEvent(self, event)
+        
+        # Harder exit to prevent segfault. Not really a solution,
+        # but it does the job until Pyside gets fixed.
+        if sys.version_info >= (3,3,0) and not restarting:
+            if hasattr(os, '_exit'):
+                os._exit(0)
     
     
     def restart(self):
         """ Restart IEP. """
         
+        self._closeflag = time.time()
+        
         # Close
         self.close()
         
-        if self._didClose:
+        if self._closeflag:
             # Get args
             args = [arg for arg in sys.argv]
             
@@ -365,7 +437,7 @@ class MainWindow(QtGui.QMainWindow):
             action.menuLauncher = tool.menuLauncher
         
         # Show menu and process result
-        a = menu.exec_(QtGui.QCursor.pos())
+        a = menu.popup(QtGui.QCursor.pos())
         if a:
             a.menuLauncher(not a.menuLauncher(None))
 
@@ -379,7 +451,7 @@ def loadAppIcons():
     
     # Determine template for filename of the application icon-files.
     # Use the Pyzo logo if in pyzo_mode.
-    if iep.pyzo_mode:
+    if False: #iep.pyzo_mode:
         fnameT = 'pyzologo{}.png'
     else:
         fnameT = 'ieplogo{}.png'
@@ -437,6 +509,30 @@ def loadIcons():
                 print('Could not load icon %s: %s' % (fname, str(err)))
 
 
+def loadFonts():
+    """ loadFonts()
+    Load all fonts that come with IEP.
+    """
+    import iep.codeeditor  # we need iep and codeeditor namespace here
+    
+    # Get directory containing the icons
+    fontDir = os.path.join(iep.iepDir, 'resources', 'fonts')
+    
+    # Get database object
+    db = QtGui.QFontDatabase()
+    
+    # Set default font
+    iep.codeeditor.Manager.setDefaultFontFamily('DejaVu Sans Mono')
+    
+    # Load fonts that are in the fonts directory
+    for fname in os.listdir(fontDir):
+        if os.path.splitext(fname)[1].lower() in ['.otf', '.ttf']:
+            try:
+                db.addApplicationFont( os.path.join(fontDir, fname) )
+            except Exception as err:
+                print('Could not load font %s: %s' % (fname, str(err)))
+
+
 class _CallbackEventHandler(QtCore.QObject):
     """ Helper class to provide the callLater function. 
     """
