@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -14,9 +14,11 @@ shortcuts.
 
 import os, sys, re, time
 import unicodedata
+import datetime
+
 from pyzolib import paths
 
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 
 import iep
 from iep.iepcore.compactTabWidget import CompactTabWidget
@@ -611,13 +613,13 @@ class EditMenu(Menu):
         self.addSeparator()
         self.addItem(translate("menu", "Find or replace ::: Show find/replace widget. Initialize with selected text."), 
             icons.find, iep.editors._findReplace.startFind)
-        self.addItem(translate("menu", "Find selection ::: Find the next occurance of the selected text."), 
+        self.addItem(translate("menu", "Find selection ::: Find the next occurrence of the selected text."), 
             None, iep.editors._findReplace.findSelection)
-        self.addItem(translate("menu", "Find selection backward ::: Find the previous occurance of the selected text."), 
+        self.addItem(translate("menu", "Find selection backward ::: Find the previous occurrence of the selected text."), 
             None, iep.editors._findReplace.findSelectionBw)
-        self.addItem(translate("menu", "Find next ::: Find the next occurance of the search string."), 
+        self.addItem(translate("menu", "Find next ::: Find the next occurrence of the search string."), 
             None, iep.editors._findReplace.findNext)
-        self.addItem(translate("menu", "Find previous ::: Find the previous occurance of the search string."), 
+        self.addItem(translate("menu", "Find previous ::: Find the previous occurrence of the search string."), 
             None, iep.editors._findReplace.findPrevious)
     
     
@@ -652,6 +654,34 @@ class ZoomMenu(Menu):
             logger.setZoom(iep.config.view.zoom)
 
 
+class FontMenu(Menu):
+    def __init__(self, parent=None, name="Font", *args, **kwds):
+        Menu.__init__(self, parent, name, *args, **kwds)
+        self.aboutToShow.connect(self._updateFonts)  
+    
+    def _updateFonts(self):
+        self.clear()
+        # Build list with known available monospace fonts
+        names = iep.codeeditor.Manager.fontNames()
+        defaultName =  'DejaVu Sans Mono'
+        for name in sorted(names):
+            txt = name+' (default)' if name == defaultName else name
+            self.addGroupItem(txt, None, self._selectFont, value=name)
+        # Select the current one
+        self.setCheckedOption(None, iep.config.view.fontname)
+    
+    def _selectFont(self, name):
+        iep.config.view.fontname = name
+        # Apply
+        for editor in iep.editors:
+            editor.setFont(iep.config.view.fontname)
+        for shell in iep.shells:
+            shell.setFont(iep.config.view.fontname)
+        logger = iep.toolManager.getTool('ieplogger')
+        if logger:
+            logger.setFont(iep.config.view.fontname)
+
+
 # todo: brace matching
 # todo: code folding?
 # todo: maybe move qt theme to settings
@@ -660,7 +690,7 @@ class ViewMenu(Menu):
         icons = iep.icons
         
         # Create edge column menu
-        t = translate("menu", "Edge Column ::: The location of the long-line-indicator.")
+        t = translate("menu", "Location of long line indicator ::: The location of the long-line-indicator.")
         self._edgeColumMenu = GeneralOptionsMenu(self, t, self._setEdgeColumn)
         values = [0] + [i for i in range(60,130,10)]
         names = ["None"] + [str(i) for i in values[1:]]
@@ -701,6 +731,7 @@ class ViewMenu(Menu):
             None, "highlightCurrentLine")
         self.addSeparator()
         self.addMenu(self._edgeColumMenu, icons.text_padding_right)
+        self.addMenu(FontMenu(self, translate("menu", "Font")), icons.style)
         self.addMenu(ZoomMenu(self, translate("menu", "Zooming")), icons.magnifier)
         self.addMenu(self._qtThemeMenu, icons.application_view_tile)
     
@@ -780,6 +811,25 @@ class ShellMenu(Menu):
                 icons.cancel, self._shellAction, "closeShell"),
             ]
     
+    def buildShellDebugActions(self):
+        """ Create the menu items for debug shell actions.
+        Returns a list of all items added"""
+        icons = iep.icons
+        
+        return [
+            self.addItem(translate("menu", 'Debug next: proceed until next line'), 
+                icons.debug_next, self._debugAction, "NEXT"),
+            self.addItem(translate("menu", 'Debug step: proceed one step'), 
+                icons.debug_step, self._debugAction, "STEP"),
+            self.addItem(translate("menu", 'Debug return: proceed until returns'), 
+                icons.debug_return, self._debugAction, "RETURN"),
+            self.addItem(translate("menu", 'Debug continue: proceed to next breakpoint'), 
+                icons.debug_continue, self._debugAction, "CONTINUE"),
+            self.addItem(translate("menu", 'Stop debugging'), 
+                icons.debug_quit, self._debugAction, "STOP"),
+            ]
+    
+    
     def getShell(self):
         """ Returns the shell on which to apply the menu actions. Default is
         the current shell, this is overridden in the shell/shell tab context
@@ -788,11 +838,28 @@ class ShellMenu(Menu):
         
     def build(self):
         """ Create the items for the shells menu """
+        
+        # Normal shell actions
         self._shellActions = self.buildShellActions()
         
         self.addSeparator()
+        
+        # Debug stuff
+        self._debug_clear_text = translate('menu', 'Clear all {} breakpoints')
+        self._debug_clear = self.addItem('', iep.icons.bug_delete, self._clearBreakPoints)
+        self._debug_pm = self.addItem(
+            translate('menu', 'Postmortem: debug from last traceback'), 
+                iep.icons.bug_delete, self._debugAction, "START")
+        self._shellDebugActions = self.buildShellDebugActions()
+        #
+        self.aboutToShow.connect(self._updateDebugButtons)
+        
+        self.addSeparator()
+        
+        # Shell config
         self.addItem(translate("menu", 'Edit shell configurations... ::: Add new shell configs and edit interpreter properties.'), 
             iep.icons.application_wrench, self._editConfig2)
+        
         self.addSeparator()
         
         # Add shell configs
@@ -810,6 +877,18 @@ class ShellMenu(Menu):
                 iep.icons.application_add, iep.shells.addShell, config)
             self._shellCreateActions.append(action)
     
+    def _updateDebugButtons(self):
+        # Count breakpoints
+        bpcount = 0
+        for e in iep.editors:
+            bpcount += len(e.breakPoints())
+        self._debug_clear.setText(self._debug_clear_text.format(bpcount))
+        # Determine state of PM and clear button
+        debugmode = iep.shells._debugmode
+        self._debug_pm.setEnabled(debugmode==0)
+        self._debug_clear.setEnabled(debugmode==0)
+        # The _shellDebugActions are enabled/disabled by the shellStack
+    
     def _shellAction(self, action):
         """ Call the method specified by 'action' on the current shell.
         """
@@ -818,6 +897,17 @@ class ShellMenu(Menu):
             # Call the specified action
             getattr(shell,action)()
     
+    def _debugAction(self, action):
+        shell = self.getShell()
+        if shell:
+            # Call the specified action
+            command = action.upper()
+            shell.executeCommand('DB %s\n' % command)
+    
+    def _clearBreakPoints(self, action=None):
+        for e in iep.editors:
+            e.clearBreakPoints()
+    
     def _editConfig2(self):
         """ Edit, add and remove configurations for the shells. """
         from iep.iepcore.shellInfoDialog import ShellInfoDialog 
@@ -826,6 +916,7 @@ class ShellMenu(Menu):
 
 
 class ShellButtonMenu(ShellMenu):
+    
     def build(self):
         self._shellActions = []
         
@@ -896,6 +987,54 @@ class ShellTabContextMenu(ShellContextMenu):
         pass
 
 
+
+class EditorContextMenu(Menu):
+    """ This is the context menu for the editor """
+    def __init__(self, editor, name='EditorContextMenu' ):
+        self._editor = editor
+        Menu.__init__(self, editor, name)
+        
+    
+    def build(self):
+        """ Build menu """
+        icons = iep.icons
+        
+        # This is a subset of the edit menu. Copied manually.
+        self.addItem(translate("menu", "Cut ::: Cut the selected text."), 
+            icons.cut, self._editItemCallback, "cut")
+        self.addItem(translate("menu", "Copy ::: Copy the selected text to the clipboard."), 
+            icons.page_white_copy, self._editItemCallback, "copy")
+        self.addItem(translate("menu", "Paste ::: Paste the text that is now on the clipboard."), 
+            icons.paste_plain, self._editItemCallback, "paste")
+        self.addItem(translate("menu", "Select all ::: Select all text."), 
+            icons.sum, self._editItemCallback, "selectAll")
+        self.addSeparator()
+        self.addItem(translate("menu", "Indent ::: Indent the selected line."), 
+            icons.text_indent, self._editItemCallback, "indentSelection")
+        self.addItem(translate("menu", "Dedent ::: Unindent the selected line."), 
+            icons.text_indent_remove, self._editItemCallback, "dedentSelection")
+        self.addItem(translate("menu", "Comment ::: Comment the selected line."), 
+            icons.comment_add, self._editItemCallback, "commentCode")
+        self.addItem(translate("menu", "Uncomment ::: Uncomment the selected line."), 
+            icons.comment_delete, self._editItemCallback, "uncommentCode")
+        self.addItem(translate("menu", "Justify comment/docstring::: Reshape the selected text so it is aligned to around 70 characters."), 
+            icons.text_align_justify, self._editItemCallback, "justifyText")
+        
+        # This is a subset of the run menu. Copied manually.
+        self.addSeparator()
+        self.addItem(translate("menu", 'Run selection ::: Run the current editor\'s selected lines, selected words on the current line, or current line if there is no selection.'), 
+            icons.run_lines, self._runSelected)
+    
+    
+    def _editItemCallback(self, action):
+        #If the widget has a 'name' attribute, call it
+        getattr(self._editor, action)()
+    
+    def _runSelected(self):
+        runMenu = iep.main.menuBar()._menumap['run']
+        runMenu._runSelected()
+
+
 class EditorTabContextMenu(Menu):
     def __init__(self, *args, **kwds):
         Menu.__init__(self, *args, **kwds)
@@ -987,6 +1126,8 @@ class RunMenu(Menu):
             icons.run_lines, self._runSelected)
         self.addItem(translate("menu", 'Run cell ::: Run the current editors\'s cell in the current shell.'), 
             icons.run_cell, self._runCell)
+        self.addItem(translate("menu", 'Run cell and advance ::: Run the current editors\'s cell and advance to the next cell.'), 
+            icons.run_cell, self._runCellAdvance)
         #In the _runFile calls, the parameter specifies (asScript, mainFile)
         self.addItem(translate("menu", 'Run file ::: Run the current file in the current shell.'), 
             icons.run_file, self._runFile,(False, False))
@@ -1004,7 +1145,7 @@ class RunMenu(Menu):
     
     def _showHelp(self):
         """ Show more information about ways to run code. """
-        from iep.iepcore.iepwizard import IEPWizard
+        from iep.util.iepwizard import IEPWizard
         w = IEPWizard(self)
         w.show('RuncodeWizardPage1') # Start wizard at page about running code
     
@@ -1055,7 +1196,7 @@ class RunMenu(Menu):
         lineNumber1 = runCursor.blockNumber()
     
         runCursor.setPosition(screenCursor.selectionEnd(),runCursor.KeepAnchor)
-        if not runCursor.atBlockStart():
+        if not (screenCursor.hasSelection() and runCursor.atBlockStart()):
             #If the end of the selection is at the beginning of a block, don't extend it
             runCursor.movePosition(runCursor.EndOfBlock,runCursor.KeepAnchor)
         lineNumber2 = runCursor.blockNumber()
@@ -1077,7 +1218,10 @@ class RunMenu(Menu):
             fname = editor.id() # editor._name or editor._filename
             shell.executeCode(code, fname, lineNumber1)
     
-    def _runCell(self):
+    def _runCellAdvance(self):
+        self._runCell(True)
+    
+    def _runCell(self, advance=False):
         """ Run the code between two cell separaters ('##'). 
         """
         #TODO: ignore ## in multi-line strings
@@ -1135,6 +1279,14 @@ class RunMenu(Menu):
         # Get filename and run code
         fname = editor.id() # editor._name or editor._filename
         shell.executeCode(code, fname, lineNumber, cellName)
+        
+        # Advance
+        if advance:
+            cursor = editor.textCursor()
+            cursor.setPosition(runCursor.position())
+            cursor.movePosition(cursor.NextBlock)
+            editor.setTextCursor(cursor)
+    
     
     def _showWhatToExecute(self, editor, runCursor=None):
         # Get runCursor for whole document if not given
@@ -1233,9 +1385,9 @@ class HelpMenu(Menu):
     
     def build(self):
         icons = iep.icons
-        
-        self.addUrlItem(translate("menu", "Pyzo Website ::: Open the Pyzo website in your browser."), 
-            icons.help, "http://www.pyzo.org")
+        if iep.pyzo_mode:
+            self.addUrlItem(translate("menu", "Pyzo Website ::: Open the Pyzo website in your browser."), 
+                icons.help, "http://www.pyzo.org")
         self.addUrlItem(translate("menu", "IEP Website ::: Open the IEP website in your browser."), 
             icons.help, "http://iep.pyzo.org")
         self.addUrlItem(translate("menu", "Ask a question ::: Need help?"), 
@@ -1245,11 +1397,14 @@ class HelpMenu(Menu):
         self.addSeparator()
         self.addItem(translate("menu", "IEP wizard ::: Get started quickly."), 
             icons.wand, self._showIepWizard)
-        self.addItem(translate("menu", "View license ::: Legal stuff."), 
-            icons.script, lambda: iep.editors.loadFile(os.path.join(iep.iepDir,"license.txt")))
+        #self.addItem(translate("menu", "View code license ::: Legal stuff."), 
+        #    icons.script, lambda: iep.editors.loadFile(os.path.join(iep.iepDir,"license.txt")))
         
         self.addItem(translate("menu", "Check for updates ::: Are you using the latest version?"), 
             icons.application_go, self._checkUpdates)
+        
+        #self.addItem(translate("menu", "Manage your IEP license ::: View/add licenses."), 
+        #    icons.script, self._manageLicenses)
         self.addItem(translate("menu", "About IEP ::: More information about IEP."), 
             icons.information, self._aboutIep)
     
@@ -1257,7 +1412,7 @@ class HelpMenu(Menu):
         self.addItem(name, icon, lambda: webbrowser.open(url))
     
     def _showIepWizard(self):
-        from iep.iepcore.iepwizard import IEPWizard
+        from iep.util.iepwizard import IEPWizard
         w = IEPWizard(self)
         w.show() # Use show() instead of exec_() so the user can interact with IEP
     
@@ -1265,19 +1420,22 @@ class HelpMenu(Menu):
         """ Check whether a newer version of IEP is available. """
         # Get versions available
         import urllib.request, re
-        url = "http://code.google.com/p/iep/downloads/list"
+        url = "http://www.iep-project.org/downloads.html"
         text = str( urllib.request.urlopen(url).read() )
         results = []
         for pattern in ['iep-(.{1,9}?)\.source\.zip' ]:
             results.extend( re.findall(pattern, text) )
         # Produce single string with all versions ...
-        versions = ', '.join(set(results))
+        def sorter(x):
+            # Tilde is high ASCII, make 3.2.1 > 3.2 and 3.2 > 3.2beta
+            return x.replace('.','~')+'~' 
+        versions = list(sorted(set(results), key=sorter, reverse=True))
         if not versions:
             versions = '?'
         # Define message
         text = "Your version of IEP is: {}\n" 
-        text += "Available versions are: {}\n\n"         
-        text = text.format(iep.__version__, versions)
+        text += "Latest available version is: {}\n\n"         
+        text = text.format(iep.__version__, versions[0])
         # Show message box
         m = QtGui.QMessageBox(self)
         m.setWindowTitle(translate("menu dialog", "Check for the latest version."))
@@ -1293,73 +1451,17 @@ class HelpMenu(Menu):
         # Goto webpage if user chose to
         if result == m.Yes:
             import webbrowser
-            webbrowser.open("http://code.google.com/p/iep/downloads/list")
+            webbrowser.open("http://www.iep-project.org/downloads.html")
+    
+    def _manageLicenses(self):
+        from iep.iepcore.license import LicenseManager
+        w = LicenseManager(None)
+        w.exec_()
+    
     
     def _aboutIep(self):
-        aboutText = """
-        <h2>IEP: the Interactive Editor for Python</h2>
-        
-        <b>Version info</b><br>
-        IEP version: <u>{}</u><br>
-        Platform: {}<br>
-        Python version: {}<br>
-        Qt version: {}<br>
-        {} version: {}<br>
-        <br>
-        
-        <b>IEP directories</b><br>
-        IEP source directory: {}<br>
-        IEP userdata directory: {}<br>
-        <br>
-        
-        <b>Acknowledgements</b><br>
-        IEP is written in Python 3 and uses the Qt widget
-        toolkit. IEP uses code and concepts that are inspired by 
-        IPython, Pype, and Spyder.
-        IEP uses a (modified) subset of the silk icon set, 
-        by Mark James (http://www.famfamfam.com/lab/icons/silk/).
-        <br><br>
-        
-        <b>License</b><br>
-        Copyright (C) 2012, the IEP development team<br>
-        IEP is distributed under the terms of the (new) BSD License.<br>
-        The full license can be found in 'license.txt'.
-        <br><br>
-        
-        <b>Developers</b><br>
-        Almar Klein<br>
-        Rob Reilink<br>
-        Ludo Visser<br>
-        
-        """
-        # Determine if this is PyQt4 or Pyside
-        if hasattr(QtCore, 'PYQT_VERSION_STR'):
-            qtWrapper = 'PyQt4'
-            qtVersion = QtCore.QT_VERSION_STR
-            qtWrapperVersion = QtCore.PYQT_VERSION_STR
-        else:
-            import PySide
-            qtWrapper = 'PySide'
-            qtVersion = QtCore.__version__
-            qtWrapperVersion = PySide.__version__
-        # Insert information texts
-        if paths.is_frozen():
-            versionText = iep.__version__ + ' (binary)'
-        else:
-            versionText = iep.__version__ + ' (source)'
-        aboutText = aboutText.format(versionText, 
-                        sys.platform, sys.version.split(' ')[0],
-                        qtVersion, qtWrapper, qtWrapperVersion,
-                        iep.iepDir, iep.appDataDir)
-        
-        # Define icon and text
-        im = QtGui.QPixmap( os.path.join(iep.iepDir,'resources', 'appicons', 'ieplogo64.png') )
-        # Show message box
-        m = QtGui.QMessageBox(self)
-        m.setTextFormat(QtCore.Qt.RichText)
-        m.setWindowTitle(translate("menu dialog", "About IEP"))
-        m.setText(unwrapText(aboutText))
-        m.setIconPixmap(im)
+        from iep.iepcore.about import AboutDialog
+        m = AboutDialog(self)
         m.exec_()
 
 
@@ -1368,6 +1470,18 @@ class SettingsMenu(Menu):
     def build(self):
         icons = iep.icons
         
+        # Create language menu
+        from iep.util.locale import LANGUAGES, LANGUAGE_SYNONYMS
+        # Update language setting if necessary
+        cur = iep.config.settings.language
+        iep.config.settings.language = LANGUAGE_SYNONYMS.get(cur, cur)
+        # Create menu        
+        t = translate("menu", "Select language ::: The language used by IEP.")
+        self._languageMenu = GeneralOptionsMenu(self, t, self._selectLanguage)
+        values = [key for key in sorted(LANGUAGES)]
+        self._languageMenu.setOptions(values, values)
+        self._languageMenu.setCheckedOption(None, iep.config.settings.language)
+        
         self.addBoolSetting(translate("menu", 'Automatically indent ::: Indent when pressing enter after a colon.'),
             'autoIndent', lambda state, key: [e.setAutoIndent(state) for e in iep.editors])
         self.addBoolSetting(translate("menu", 'Enable calltips ::: Show calltips with function signatures.'), 
@@ -1382,6 +1496,7 @@ class SettingsMenu(Menu):
             icons.keyboard, lambda: KeymappingDialog().exec_())
         self.addItem(translate("menu", 'Edit syntax styles... ::: Change the coloring of your code.'), 
             icons.style, self._editStyles)
+        self.addMenu(self._languageMenu, icons.flag_green)
         self.addItem(translate("menu", 'Advanced settings... ::: Configure IEP even further.'), 
             icons.cog, self._advancedSettings)
     
@@ -1432,6 +1547,23 @@ class SettingsMenu(Menu):
         self.addCheckItem(name, None, _callback, key, 
             getattr(iep.config.settings,key)) #Default value
     
+    def _selectLanguage(self, languageName):
+        # Skip if the same
+        if iep.config.settings.language == languageName:
+            return
+        # Save new language
+        iep.config.settings.language = languageName
+        # Notify user
+        text = translate('menu dialog', """
+        The language has been changed. 
+        IEP needs to restart for the change to take effect.
+        """)
+        m = QtGui.QMessageBox(self)
+        m.setWindowTitle(translate("menu dialog", "Language changed"))
+        m.setText(unwrapText(text))
+        m.setIcon(m.Information)
+        m.exec_()
+
 
 # Remains of old settings menu. Leave here because some settings should some day be 
 # accessible via a dialog (advanced settings).
@@ -1575,7 +1707,7 @@ class KeyMapModel(QtCore.QAbstractItemModel):
             if not value:
                 value = '-'*10
             elif index.column()>0:
-                key1, key2 = '<>','<>'
+                key1, key2 = ' ', ' '
                 shortcuts = getShortcut(item)
                 if shortcuts[0]:
                     key1 = shortcuts[0]
@@ -1586,21 +1718,25 @@ class KeyMapModel(QtCore.QAbstractItemModel):
         key2 = translateShortcutToOSNames(key2)
         
         # obtain value
-        value = [value,key1,key2][index.column()]
+        value = [value,key1,key2, ''][index.column()]
         
         # return
         if role == 0:
             # display role
             return value
+        elif role == 8:
+            # 8: BackgroundRole
+            if not value:
+                return None
+            elif index.column() == 1:
+                return QtGui.QBrush(QtGui.QColor(200,220,240))
+            elif index.column() == 2:
+                return QtGui.QBrush(QtGui.QColor(210,230,250))
+            else:
+                return None
         else:
             return None
-#             # 8: BackgroundRole
-#             brush = QtGui.QBrush(QtGui.QColor(240,255,240))
-#             if value and index.column()>0:
-#                 return brush
-#             else:
-#                 return None
-            
+    
     
     def rowCount(self, parent):
         if parent.isValid():
@@ -1610,11 +1746,11 @@ class KeyMapModel(QtCore.QAbstractItemModel):
             return len(self._root.actions())
     
     def columnCount(self, parent):
-        return 3
+        return 4
     
     def headerData(self, section, orientation, role):
         if role == 0:# and orientation==1:
-            tmp = ['Menu action','Shortcut 1','Shortcut 2']
+            tmp = ['Menu action','Shortcut 1','Shortcut 2', '']
             return tmp[section]
     
     def parent(self, index):
@@ -1780,7 +1916,7 @@ class KeyMapEditDialog(QtGui.QDialog):
         
         self._label = QtGui.QLabel("", self)
         self._label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)
-        self._label.resize(size[0]-20, 80)
+        self._label.resize(size[0]-20, 100)
         self._label.move(10,2)
         
         self._line = KeyMapLineEdit('', self)
@@ -1854,11 +1990,10 @@ class KeyMapEditDialog(QtGui.QDialog):
             elif shortcuts[1].lower() == shortcut.lower():
                 primSec = 'secondary'
             # if a correspondence, let the user know
-            if primSec:
-                tmp = "WARNING: combo already in use "
-                tmp += "as "+primSec+" shortcut for:\n" 
+            if primSec and key != self._fullname:
+                tmp = "Warning: shortcut already in use for:\n"
                 tmp += key.replace('__',' -> ').replace('_', ' ')
-                self._label.setText(self._intro + '\n\n' + tmp)
+                self._label.setText(self._intro + '\n\n' + tmp + '\n')
                 break
         else:
             self._label.setText(self._intro)
