@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -16,7 +16,7 @@ code in it.
 
 """
 
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 Qt = QtCore.Qt
 
 import os, sys, time, subprocess
@@ -96,13 +96,16 @@ def finishKernelInfo(info, scriptFile=None):
     else:
         info.scriptFile = ''
     
-    #If the project manager is active, and has the check box
+    #If the file browser is active, and has the check box
     #'add path to Python path' set, set the PROJECTPATH variable
+    fileBrowser = iep.toolManager.getTool('iepfilebrowser')
     projectManager = iep.toolManager.getTool('iepprojectmanager')
-    if projectManager:
+    info.projectPath = ''
+    if fileBrowser:
+        info.projectPath = fileBrowser.getAddToPythonPath()
+    if projectManager and not info.projectPath:
+        # Only process project manager tool if file browser did not set a path.
         info.projectPath = projectManager.getAddToPythonPath()
-    else:
-        info.projectPath = ''
     
     return info
 
@@ -213,7 +216,8 @@ class BaseShell(BaseTextCtrl):
     
     def __init__(self, parent,**kwds):
         super().__init__(parent, wrap=True, showLineNumbers=False, 
-            highlightCurrentLine=False, parser='python', **kwds)
+            showBreakPoints=False, highlightCurrentLine=False, parser='python', 
+            **kwds)
         
         # Use a special highlighter that only highlights the input.
         self._setHighlighter(ShellHighlighter)
@@ -310,17 +314,22 @@ class BaseShell(BaseTextCtrl):
     def keyPressEvent(self,event):
         
         if event.key() in [Qt.Key_Return, Qt.Key_Enter]:
-            # Enter: execute line
-            # Remove calltip and autocomp if shown
-            self.autocompleteCancel()
-            self.calltipCancel()
             
-            # reset history needle
-            self._historyNeedle = None
-            
-            # process
-            self.processLine()
-            return
+            # First check if autocompletion triggered
+            if self.potentiallyAutoComplete(event):
+                return
+            else:
+                # Enter: execute line
+                # Remove calltip and autocomp if shown
+                self.autocompleteCancel()
+                self.calltipCancel()
+                
+                # reset history needle
+                self._historyNeedle = None
+                
+                # process
+                self.processLine()
+                return
         
         if event.key() == Qt.Key_Escape:
             # Escape clears command
@@ -739,7 +748,10 @@ class PythonShell(BaseShell):
         # Add context menu
         self._menu = ShellContextMenu(shell=self, parent=self)
         self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
-        self.customContextMenuRequested.connect(lambda p: self._menu.exec_(self.mapToGlobal(p))) 
+        self.customContextMenuRequested.connect(lambda p: self._menu.popup(self.mapToGlobal(p+QtCore.QPoint(0,3)))) 
+        
+        # Keep track of breakpoints
+        iep.editors.breakPointsChanged.connect(self.sendBreakPoints)
         
         # Start!
         self.resetVariables()
@@ -803,6 +815,7 @@ class PythonShell(BaseShell):
         self._stat_debug = yoton.StateChannel(ct, 'stat-debug', yoton.OBJECT)
         self._stat_startup = yoton.StateChannel(ct, 'stat-startup', yoton.OBJECT)
         self._stat_startup.received.bind(self._onReceivedStartupInfo)
+        self._stat_breakpoints = yoton.StateChannel(ct, 'stat-breakpoints', yoton.OBJECT)
         
         # Create introspection request channel
         self._request = yoton.ReqChannel(ct, 'reqp-introspect')
@@ -1067,6 +1080,13 @@ class PythonShell(BaseShell):
         self._ctrl_code.send(msg)
     
     
+    def sendBreakPoints(self, breaks):
+        """ Send all breakpoints. 
+        """
+        # breaks is a dict of filenames to integers
+        self._stat_breakpoints.send(breaks)
+    
+    
     ## The polling methods and terminating methods
     
     def poll(self, channel=None):
