@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -13,7 +13,7 @@ and a dialog to edit the shell configurations.
 """
 
 import os, sys, time, re
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 
 import iep
 from iep import translate
@@ -97,7 +97,8 @@ class ShellStackWidget(QtGui.QWidget):
         
         # Populate toolbar
         self._shellButton = ShellControl(self._toolbar, self._stack)
-        self._dbc = DebugControl(self._toolbar)
+        self._debugmode = 0
+        self._dbs = DebugStack(self._toolbar)
         #
         self._toolbar.addWidget(self._shellButton)
         self._toolbar.addSeparator()
@@ -182,11 +183,20 @@ class ShellStackWidget(QtGui.QWidget):
         """
         
         if shell is self.getCurrentShell():
+            
             # Update debug info
             if shell and shell._debugState:
-                self._dbc.setTrace(shell._debugState)
+                info = shell._debugState
+                self._debugmode = info['debugmode']
+                for action in self._debugActions:
+                    action.setEnabled(self._debugmode==2)
+                self._debugActions[-1].setEnabled(self._debugmode>0)  # Stop
+                self._dbs.setTrace(shell._debugState)
             else:
-                self._dbc.setTrace(None)
+                for action in self._debugActions:
+                    action.setEnabled(False)
+                self._debugmode = 0
+                self._dbs.setTrace(None)
             # Send signal
             self.currentShellStateChanged.emit()
     
@@ -239,9 +249,17 @@ class ShellStackWidget(QtGui.QWidget):
         # Add actions
         for action in iep.main.menuBar()._menumap['shell']._shellActions:
             action = self._toolbar.addAction(action)
-        # Delayed-add debug control button
-        self._toolbar.addWidget(self._dbc)
-    
+        
+        self._toolbar.addSeparator()
+        
+        # Add debug actions
+        self._debugActions = []
+        for action in iep.main.menuBar()._menumap['shell']._shellDebugActions:
+            self._debugActions.append(action)
+            action = self._toolbar.addAction(action)
+        
+        # Delayed-add debug control buttons
+        self._toolbar.addWidget(self._dbs)
     
     def contextMenuTriggered(self, p):
         """ Called when context menu is clicked """
@@ -251,7 +269,7 @@ class ShellStackWidget(QtGui.QWidget):
         
         if shell:
             p = self._shellButton.mapToGlobal(self._shellButton.rect().bottomLeft())
-            ShellTabContextMenu(shell=shell, parent=self).exec_(p)
+            ShellTabContextMenu(shell=shell, parent=self).popup(p)
     
     
     def onShellAction(self, action):
@@ -348,39 +366,124 @@ class ShellControl(QtGui.QToolButton):
 
 
 
-class DebugControl(QtGui.QToolButton):
-    """ A button that can be used for post mortem debuggin. 
+# todo: remove this?
+# class DebugControl(QtGui.QToolButton):
+#     """ A button to control debugging. 
+#     """
+#     
+#     def __init__(self, parent):
+#         QtGui.QToolButton.__init__(self, parent)
+#         
+#         # Flag
+#         self._debugmode = False
+#         
+#         # Set text
+#         self.setText(translate('debug', 'Debug'))
+#         self.setIcon(iep.icons.bug)
+#         self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+#         #self.setPopupMode(self.InstantPopup)
+#         
+#         # Bind to triggers
+#         self.triggered.connect(self.onTriggered)
+#         self.pressed.connect(self.onPressed)
+#         self.buildMenu()
+#     
+#     
+#     def buildMenu(self):
+#         
+#         # Count breakpoints
+#         bpcount = 0
+#         for e in iep.editors:
+#             bpcount += len(e.breakPoints())
+#         
+#         # Prepare a text
+#         clearallbps = translate('debug', 'Clear all {} breakpoints')
+#         clearallbps = clearallbps.format(bpcount)
+#         
+#         # Set menu
+#         menu = QtGui.QMenu(self)
+#         self.setMenu(menu)
+#         
+#         for cmd, enabled, icon, text in [ 
+#                 ('CLEAR', self._debugmode==0, iep.icons.bug_delete, clearallbps),
+#                 ('PM', self._debugmode==0, iep.icons.bug_error, 
+#                     translate('debug', 'Postmortem: debug from last traceback')),
+#                 ('STOP', self._debugmode>0, iep.icons.debug_quit, 
+#                     translate('debug', 'Stop debugging')),
+# #                 ('NEXT', self._debugmode==2, iep.icons.debug_next, 
+# #                     translate('debug', 'Next: proceed until next line')),
+# #                 ('STEP', self._debugmode==2, iep.icons.debug_step, 
+# #                     translate('debug', 'Step: proceed one step')),
+# #                 ('RETURN', self._debugmode==2, iep.icons.debug_return, 
+# #                     translate('debug', 'Return: proceed until returns')),
+# #                 ('CONTINUE', self._debugmode==2, iep.icons.debug_continue, 
+# #                     translate('debug', 'Continue: proceed to next breakpoint')),
+#                 ]:
+#             if cmd is None:
+#                 menu.addSeparator()
+#             else:
+#                 if icon is not None:
+#                     a = menu.addAction(icon, text)
+#                 else:
+#                     a = menu.addAction(text)
+#                 if hasattr(text, 'tt'):
+#                     a.setToolTip(text.tt)    
+#                 a.cmd = cmd
+#                 a.setEnabled(enabled)
+#     
+#     
+#     def onPressed(self, show=True):
+#         self.buildMenu()
+#         self.showMenu()
+#     
+#     
+#     def onTriggered(self, action):
+#         if action.cmd == 'PM':  
+#             # Initiate postmortem debugging
+#             shell = iep.shells.getCurrentShell()
+#             if shell:
+#                 shell.executeCommand('DB START\n')
+#         
+#         elif action.cmd == 'CLEAR':
+#             # Clear all breakpoints
+#             for e in iep.editors:
+#                 e.clearBreakPoints()
+#         
+#         else:
+#             command = action.cmd.upper()
+#             shell = iep.shells.getCurrentShell()
+#             if shell:
+#                 shell.executeCommand('DB %s\n' % command)
+#     
+#     
+#     def setTrace(self, info):
+#         """ Determine whether we are in debug mode. 
+#         """
+#         if info is None:
+#             self._debugmode = 0
+#         else:
+#             self._debugmode = info['debugmode']
+
+
+
+class DebugStack(QtGui.QToolButton):
+    """ A button that shows the stack trace.
     """
     
     def __init__(self, parent):
         QtGui.QToolButton.__init__(self, parent)
         
         # Set text and tooltip
-        self.setText('Debug')
-        self.setIcon(iep.icons.bug)
+        self._baseText = translate('debug', 'Stack')
+        self.setText('%s:' % self._baseText)
+        self.setIcon(iep.icons.text_align_justify)
         self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
-        self.setToolTip("Start/Stop post mortem debugging.")
-        
-        # Set mode
         self.setPopupMode(self.InstantPopup)
         
         # Bind to triggers
-        self.pressed.connect(self.onPressed)
         self.triggered.connect(self.onTriggered)
     
     
-    def onPressed(self):
-        # Also fires after clicking on an action and (if there's a
-        # menu) clicking outside the button and menu 
-        
-        if not self.menu():
-            
-            # Initiate debugging
-            shell = iep.shells.getCurrentShell()
-            if shell:
-                shell.executeCommand('DB START\n')
-    
-    
     def onTriggered(self, action):
         
         # Get shell
@@ -388,17 +491,13 @@ class DebugControl(QtGui.QToolButton):
         if not shell:
             return
         
-        if action._index < 1:
-            # Stop debugging
-            shell.executeCommand('DB STOP\n')
-        else:
-            # Change stack index
-            if not action._isCurrent:
-                shell.executeCommand('DB FRAME {}\n'.format(action._index))
-            # Open file and select line
-            if True:
-                line = action.text().split(': ',1)[1]
-                self.debugFocus(line)
+        # Change stack index
+        if not action._isCurrent:
+            shell.executeCommand('DB FRAME {}\n'.format(action._index))
+        # Open file and select line
+        if True:
+            line = action.text().split(': ',1)[1]
+            self.debugFocus(line)
     
     
     def setTrace(self, info):
@@ -410,15 +509,17 @@ class DebugControl(QtGui.QToolButton):
         
         # Get info
         if info:
-            index, frames = info['index'], info['frames']
+            index, frames, debugmode = info['index'], info['frames'], info['debugmode']
         else:
             index, frames = -1, []
         
-        if not frames:
+        if (not frames) or (debugmode==0):
             
             # Remove trace
             self.setMenu(None)
-            self.setText('Debug')
+            self.setText('')  #(self._baseText)
+            self.setEnabled(False)
+            iep.editors.setDebugLineIndicator('', 0)
         
         else:
             # Get the current frame
@@ -427,27 +528,31 @@ class DebugControl(QtGui.QToolButton):
             # Create menu and add __main__
             menu = QtGui.QMenu(self)
             self.setMenu(menu)
-            action = menu.addAction('MAIN: stop debugging')
-            action._index = 0
             
             # Fill trace
             for i in range(len(frames)):
                 thisIndex = i + 1
-                action = menu.addAction('{}: {}'.format(thisIndex, frames[i]))
+                text = '{}: File "{}", line {}, in {}'
+                text = text.format(thisIndex, *frames[i])
+                action = menu.addAction(text)
                 action._index = thisIndex
                 action._isCurrent = False
                 if thisIndex == index:
                     action._isCurrent = True
                     theAction = action
-                    
+                    # Send notice to editor stack
+                    filename, linenr, func = frames[i]
+                    iep.editors.setDebugLineIndicator(filename, linenr)
             
             # Highlight current item and set the button text
             if theAction:
                 menu.setDefaultAction(theAction)
                 #self.setText(theAction.text().ljust(20))
                 i = theAction._index
-                text = "Stack Trace ({}/{}):  ".format(i, len(frames))
+                text = "{} ({}/{}):  ".format(self._baseText, i, len(frames))
                 self.setText(text)
+            
+            self.setEnabled(True)
     
     
     def debugFocus(self, lineFromDebugState):
@@ -477,3 +582,4 @@ class DebugControl(QtGui.QToolButton):
             cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
             editor.setTextCursor(cursor)
 
+
